<?php

namespace common\logic\Distribution\Distribution\Service;

use common\logic\AbstractGenerator;
use common\logic\Distribution\Distribution\Model\Distribution;
use common\logic\Distribution\ProductDistribution\Repository\ProductDistributionRepository;
use common\logic\Order\Order\Repository\OrderRepository;
use common\logic\Order\Order\Service\OrderBuilder;
use common\logic\PointSale\PointSale\Repository\PointSaleRepository;
use common\logic\Product\Product\Repository\ProductRepository;
use kartik\mpdf\Pdf;

class DistributionReportPdfGenerator extends AbstractGenerator
{
    protected OrderRepository $orderRepository;
    protected OrderBuilder $orderBuilder;
    protected ProductRepository $productRepository;
    protected ProductDistributionRepository $productDistributionRepository;
    protected PointSaleRepository $pointSaleRepository;

    public function loadDependencies(): void
    {
        $this->orderRepository = $this->loadService(OrderRepository::class);
        $this->orderBuilder = $this->loadService(OrderBuilder::class);
        $this->productRepository = $this->loadService(ProductRepository::class);
        $this->productDistributionRepository = $this->loadService(ProductDistributionRepository::class);
        $this->pointSaleRepository = $this->loadService(PointSaleRepository::class);
    }

    public function generateDistributionReportPdf(Distribution $distribution, bool $save = false)
    {
        $producer = $this->getProducerContext();
        $ordersArray = $this->orderRepository->findOrdersByDistribution($distribution);
        $productsArray = $this->productRepository->findProductsByDistribution($distribution);
        $selectedProductsArray = $this->productDistributionRepository->findProductDistributionsByDistribution($distribution);
        $pointsSaleArray = $this->pointSaleRepository->findPointSales();
        foreach ($pointsSaleArray as $pointSale) {
            $this->orderBuilder->initPointSaleOrders($pointSale, $ordersArray);
        }

        $viewPdf = '@backend/views/distribution/report';
        $orientationPdf = Pdf::ORIENT_PORTRAIT;

        if ($producer->slug == 'bourlinguepacotille') {
            $viewPdf = '@backend/views/distribution/report-bourlingue';
            $orientationPdf = Pdf::ORIENT_LANDSCAPE;
        }

        // get your HTML raw content without any layouts or scripts
        $content = \Yii::$app->getView()->render($viewPdf, [
            'date' => $distribution->date,
            'distribution' => $distribution,
            'selectedProductsArray' => $selectedProductsArray,
            'pointsSaleArray' => $pointsSaleArray,
            'productsArray' => $productsArray,
            'ordersArray' => $ordersArray,
            'producer' => $producer
        ]);

        $dateStr = date('d/m/Y', strtotime($distribution->date));

        if ($save) {
            $destination = Pdf::DEST_FILE;
        } else {
            $destination = Pdf::DEST_BROWSER;
        }

        $pdf = new Pdf([
            'mode' => Pdf::MODE_UTF8,
            'format' => Pdf::FORMAT_A4,
            'orientation' => $orientationPdf,
            'destination' => $destination,
            'filename' => \Yii::getAlias(
                '@app/web/pdf/Commandes-' . $distribution->date . '-' . $this->getProducerContextId() . '.pdf'
            ),
            'content' => $content,
            //'cssFile' => Yii::getAlias('@web/css/distribution/report.css'),
            'cssInline' => '
                        table {
                            border-spacing : 0px ;
                            border-collapse : collapse ;
                            width: 100% ;
                        }
                        
                        table tr th,
                        table tr td {
                            padding: 0px ;
                            margin: 0px ;
                            border: solid 1px #e0e0e0 ;
                            padding: 3px 8px ;
                            vertical-align : top;
                            page-break-inside: avoid !important;
                            color: black;
                        }

                        table tr th { 
                            font-size: 13px ;
                        }

                        table tr td { 
                            font-size: 13px ;
                        }
                    ',
            'methods' => [
                'SetHeader' => ['Commandes du ' . $dateStr],
                'SetFooter' => ['{PAGENO}'],
            ]
        ]);

        return $pdf->render();
    }
}